@@ -22,7 +22,8 @@
 use super::BacktrackingRecorder;
 use super::BacktrackingState::*;
 
-/// A wrapper around an existing iterator to extend it with backtracking functionality
+/// An iterator over a historical record which produces memory clones of historical
+/// elements
 pub struct CopyingBacktrackingIterator<'record, I> where I: Iterator, I::Item: Clone {
   recorder: &'record mut BacktrackingRecorder<I>,
 }
@@ -60,7 +61,6 @@ impl<'record, I> Iterator for CopyingBacktrackingIterator<'record, I> where I: I
 }
 
 impl<'record, I> CopyingBacktrackingIterator<'record, I> where I:Iterator, I::Item: Clone {
-  /// Create a `CopyingBacktrackingIterator` from an existing iterator.
   pub(crate) fn new(recorder: &'record mut BacktrackingRecorder<I>) -> Self {
     CopyingBacktrackingIterator {
       recorder,
@@ -88,19 +88,6 @@ impl<'record, I> BacktrackingIterator for CopyingBacktrackingIterator<'record, I
   fn backtrack(&mut self, position: usize) {
     self.recorder.state = Backtracking { position };
   }
-
-  fn forget_before(&mut self, position: usize) {
-    if position <= self.recorder.backtracking_vec.len() {
-      //Split the history at the given point
-      let kept = self.recorder.backtracking_vec.split_off(position);
-      //Keep the second half
-      self.recorder.backtracking_vec = kept;
-    }
-  }
-
-  fn forget(&mut self) {
-    self.recorder.backtracking_vec.clear();
-  }
 }
 
 use super::Walkbackable;
@@ -116,11 +103,8 @@ impl<'history, 'record, I: 'history> Walkbackable<'history> for CopyingBacktrack
   }
 }
 
-/// A backwalk through a `BacktrackingIterator`'s history. Yields references
-/// to items in the history, and can be used to walk back to a desired point.
-/// The current position is before the most-recently-yielded element. To restart
-/// a `BacktrackingIterator` at the current position of the backwalk, use the
-/// `history_position()` method.
+/// A backwalk through a `CopyingBacktrackingIterator`'s history. Yields items in 
+/// the history, and can be used to walk back to a desired point.
 pub struct CopyingWalkback<'record, I> where I: Iterator, I::Item: Clone {
   backtracker: &'record BacktrackingRecorder<I>,
   reverse_position: usize,
